@@ -34,7 +34,7 @@ static OrcTarget *default_target;
 #define ORC_UW_MIN 0
 #define ORC_SL_MAX 2147483647
 #define ORC_SL_MIN (-1-ORC_SL_MAX)
-#define ORC_UL_MAX 4294967295ULL
+#define ORC_UL_MAX ORC_UINT64_C(4294967295)
 #define ORC_UL_MIN 0
 
 #define ORC_CLAMP_SB(x) ORC_CLAMP(x,ORC_SB_MIN,ORC_SB_MAX)
@@ -397,6 +397,17 @@ static OrcStaticOpcode opcodes[] = {
   { "storeq", ORC_STATIC_OPCODE_STORE, { 8 }, { 8 }, emulate_storeq },
   { "splatw3q", 0, { 8 }, { 8 }, emulate_splatw3q },
   { "copyq", 0, { 8 }, { 8 }, emulate_copyq },
+  { "cmpeqq", 0, { 8 }, { 8, 8 }, emulate_cmpeqq },
+  { "cmpgtsq", 0, { 8 }, { 8, 8 }, emulate_cmpgtsq },
+  { "andq", 0, { 8 }, { 8, 8 }, emulate_andq },
+  { "andnq", 0, { 8 }, { 8, 8 }, emulate_andnq },
+  { "orq", 0, { 8 }, { 8, 8 }, emulate_orq },
+  { "xorq", 0, { 8 }, { 8, 8 }, emulate_xorq },
+  { "addq", 0, { 8 }, { 8, 8 }, emulate_addq },
+  { "subq", 0, { 8 }, { 8, 8 }, emulate_subq },
+  { "shlq", ORC_STATIC_OPCODE_SCALAR, { 8 }, { 8, 8 }, emulate_shlq },
+  { "shrsq", ORC_STATIC_OPCODE_SCALAR, { 8 }, { 8, 8 }, emulate_shrsq },
+  { "shruq", ORC_STATIC_OPCODE_SCALAR, { 8 }, { 8, 8 }, emulate_shruq },
 
   { "convsbw", 0, { 2 }, { 1 }, emulate_convsbw },
   { "convubw", 0, { 2 }, { 1 }, emulate_convubw },
@@ -432,10 +443,8 @@ static OrcStaticOpcode opcodes[] = {
   { "mulubw", 0, { 2 }, { 1, 1 }, emulate_mulubw },
   { "mulswl", 0, { 4 }, { 2, 2 }, emulate_mulswl },
   { "muluwl", 0, { 4 }, { 2, 2 }, emulate_muluwl },
-#ifdef ENABLE_64BIT
   { "mulslq", 0, { 8 }, { 4, 4 }, emulate_mulslq },
   { "mululq", 0, { 8 }, { 4, 4 }, emulate_mululq },
-#endif
 
   /* accumulators */
   { "accw", ORC_STATIC_OPCODE_ACCUMULATOR, { 2 }, { 2 }, emulate_accw },
@@ -444,11 +453,16 @@ static OrcStaticOpcode opcodes[] = {
 
   { "swapw", 0, { 2 }, { 2 }, emulate_swapw },
   { "swapl", 0, { 4 }, { 4 }, emulate_swapl },
+  { "swapwl", 0, { 4 }, { 4 }, emulate_swapwl },
   { "swapq", 0, { 8 }, { 8 }, emulate_swapq },
+  { "swaplq", 0, { 8 }, { 8 }, emulate_swaplq },
   { "select0wb", 0, { 1 }, { 2 }, emulate_select0wb },
   { "select1wb", 0, { 1 }, { 2 }, emulate_select1wb },
   { "select0lw", 0, { 2 }, { 4 }, emulate_select0lw },
   { "select1lw", 0, { 2 }, { 4 }, emulate_select1lw },
+  { "select0ql", 0, { 4 }, { 8 }, emulate_select0ql },
+  { "select1ql", 0, { 4 }, { 8 }, emulate_select1ql },
+  { "mergelq", 0, { 8 }, { 4, 4 }, emulate_mergelq },
   { "mergewl", 0, { 4 }, { 2, 2 }, emulate_mergewl },
   { "mergebw", 0, { 2 }, { 1, 1 }, emulate_mergebw },
   { "splitql", 0, { 4, 4 }, { 8 }, emulate_splitql },
