@@ -76,7 +76,11 @@ powerpc_rule_loadpX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
           powerpc_emit_VX(compiler, 0x1000030c,
               powerpc_regnum(dest->alloc), value & 0x1f, 0);
         } else {
-          ORC_COMPILER_ERROR(compiler,"can't load constant");
+          value &= 0xff;
+          value |= value << 8;
+          value |= value << 16;
+          powerpc_load_long_constant (compiler, dest->alloc, value, value,
+              value, value);
         }
         break;
       case 2:
@@ -86,7 +90,10 @@ powerpc_rule_loadpX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
           powerpc_emit_VX(compiler, 0x1000034c,
               powerpc_regnum(dest->alloc), value & 0x1f, 0);
         } else {
-          ORC_COMPILER_ERROR(compiler,"can't load constant");
+          value &= 0xffff;
+          value |= value << 16;
+          powerpc_load_long_constant (compiler, dest->alloc, value, value,
+              value, value);
         }
         break;
       case 4:
@@ -96,7 +103,8 @@ powerpc_rule_loadpX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
           powerpc_emit_VX(compiler, 0x1000038c,
               powerpc_regnum(dest->alloc), value & 0x1f, 0);
         } else {
-          ORC_COMPILER_ERROR(compiler,"can't load constant");
+          powerpc_load_long_constant (compiler, dest->alloc, value, value,
+              value, value);
         }
         break;
     }
@@ -155,6 +163,75 @@ powerpc_rule_loadX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
   powerpc_emit_vperm (compiler, dest->alloc, dest->alloc, dest->alloc, perm);
 }
 
+static void
+powerpc_rule_loadoffX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
+{
+  OrcVariable *src = compiler->vars + insn->src_args[0];
+  OrcVariable *dest = compiler->vars + insn->dest_args[0];
+  int size = src->size << compiler->loop_shift;
+  int perm = orc_compiler_get_temp_reg (compiler);
+  int offset;
+
+  if (compiler->vars[insn->src_args[1]].vartype != ORC_VAR_TYPE_CONST) {
+    ORC_COMPILER_ERROR(compiler, "Rule only works with consts");
+    return;
+  }
+
+  offset = compiler->vars[insn->src_args[1]].value.i * src->size;
+  powerpc_emit_addi (compiler, compiler->gp_tmpreg, POWERPC_R0, offset);
+  switch (size) {
+    case 1:
+      ORC_ASM_CODE(compiler,"  lvebx %s, %s, %s\n",
+          powerpc_get_regname (dest->alloc),
+          powerpc_get_regname (compiler->gp_tmpreg),
+          powerpc_get_regname (src->ptr_register));
+      powerpc_emit_X (compiler, 0x7c00000e, powerpc_regnum(dest->alloc),
+          powerpc_regnum(compiler->gp_tmpreg),
+          powerpc_regnum(src->ptr_register));
+      break;
+    case 2:
+      ORC_ASM_CODE(compiler,"  lvehx %s, %s, %s\n",
+          powerpc_get_regname (dest->alloc),
+          powerpc_get_regname (compiler->gp_tmpreg),
+          powerpc_get_regname (src->ptr_register));
+      powerpc_emit_X (compiler, 0x7c00004e, powerpc_regnum(dest->alloc),
+          powerpc_regnum(compiler->gp_tmpreg),
+          powerpc_regnum(src->ptr_register));
+      break;
+    case 4:
+      ORC_ASM_CODE(compiler,"  lvewx %s, %s, %s\n",
+          powerpc_get_regname (dest->alloc),
+          powerpc_get_regname (compiler->gp_tmpreg),
+          powerpc_get_regname (src->ptr_register));
+      powerpc_emit_X (compiler, 0x7c00008e, powerpc_regnum(dest->alloc),
+          powerpc_regnum(compiler->gp_tmpreg),
+          powerpc_regnum(src->ptr_register));
+      break;
+    case 8:
+    case 16:
+      ORC_ASM_CODE(compiler,"  lvx %s, %s, %s\n",
+          powerpc_get_regname (dest->alloc),
+          powerpc_get_regname (compiler->gp_tmpreg),
+          powerpc_get_regname (src->ptr_register));
+      powerpc_emit_X (compiler, 0x7c0000ce, powerpc_regnum(dest->alloc),
+          powerpc_regnum(compiler->gp_tmpreg),
+          powerpc_regnum(src->ptr_register));
+      break;
+    default:
+      ORC_COMPILER_ERROR(compiler,"bad load size %d",
+          src->size << compiler->loop_shift);
+      break;
+  }
+  ORC_ASM_CODE(compiler,"  lvsl %s, %s, %s\n",
+      powerpc_get_regname (perm),
+      powerpc_get_regname (compiler->gp_tmpreg),
+      powerpc_get_regname (src->ptr_register));
+  powerpc_emit_X (compiler, 0x7c00000c, powerpc_regnum(perm),
+      powerpc_regnum(compiler->gp_tmpreg),
+      powerpc_regnum(src->ptr_register));
+  powerpc_emit_vperm (compiler, dest->alloc, dest->alloc, dest->alloc, perm);
+}
+
 static void
 powerpc_rule_storeX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
 {
@@ -257,11 +334,8 @@ powerpc_rule_ ## name (OrcCompiler *p, void *user, OrcInstruction *insn) \
     powerpc_emit_VX(p, 0x1000030c, \
         powerpc_regnum(p->tmpreg), (int)p->vars[insn->src_args[1]].value.i, 0); \
     powerpc_emit_VX_2 (p, opcode, code , dest, src1, p->tmpreg);\
-  } else if (p->vars[insn->src_args[1]].vartype == ORC_VAR_TYPE_PARAM) { \
-    ORC_COMPILER_ERROR(p,"rule only works with constants"); \
-    powerpc_emit_VX_2 (p, opcode, code , dest, src1, src2);\
   } else { \
-    ORC_COMPILER_ERROR(p,"rule only works with constants or params"); \
+    powerpc_emit_VX_2 (p, opcode, code , dest, src1, src2);\
   } \
 }
 
@@ -331,6 +405,10 @@ RULE(subssl, "vsubsws", 0x10000780)
 RULE(subusl, "vsubuws", 0x10000680)
 RULE(xorl, "vxor", 0x100004c4)
 
+RULE(andq, "vand", 0x10000404)
+RULE(orq, "vor", 0x10000484)
+RULE(xorq, "vxor", 0x100004c4)
+
 RULE(addf, "vaddfp", 0x1000000a)
 RULE(subf, "vsubfp", 0x1000004a)
 RULE(maxf, "vmaxfp", 0x1000040a)
@@ -569,8 +647,6 @@ powerpc_rule_muluwl (OrcCompiler *p, void *user, OrcInstruction *insn)
   powerpc_emit_vmuleuh (p, dest, src1, src2);
 }
 
-#if 0
-/* doesn't work */
 static void
 powerpc_rule_accw (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
@@ -579,7 +655,6 @@ powerpc_rule_accw (OrcCompiler *p, void *user, OrcInstruction *insn)
 
   powerpc_emit_vadduhm (p, dest, dest, src1);
 }
-#endif
 
 static void
 powerpc_rule_accl (OrcCompiler *p, void *user, OrcInstruction *insn)
@@ -590,8 +665,6 @@ powerpc_rule_accl (OrcCompiler *p, void *user, OrcInstruction *insn)
   powerpc_emit_vadduwm (p, dest, dest, src1);
 }
 
-#if 0
-/* doesn't work */
 static void
 powerpc_rule_accsadubl (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
@@ -603,10 +676,21 @@ powerpc_rule_accsadubl (OrcCompiler *p, void *user, OrcInstruction *insn)
 
   powerpc_emit_vmaxub (p, tmp1, src1, src2);
   powerpc_emit_vminub (p, tmp2, src1, src2);
+
   powerpc_emit_vsububm (p, tmp1, tmp1, tmp2);
-  powerpc_emit_vsum4ubs (p, dest, dest, tmp1);
+  if (p->loop_shift == 0) {
+    powerpc_emit_vxor (p, tmp2, tmp2, tmp2);
+    powerpc_emit_vmrghb (p, tmp1, tmp2, tmp1);
+    powerpc_emit_vmrghh (p, tmp1, tmp2, tmp1);
+    powerpc_emit_vadduwm (p, dest, dest, tmp1);
+  } else if (p->loop_shift == 1) {
+    powerpc_emit_vxor (p, tmp2, tmp2, tmp2);
+    powerpc_emit_vmrghh (p, tmp1, tmp2, tmp1);
+    powerpc_emit_vsum4ubs (p, dest, dest, tmp1);
+  } else {
+    powerpc_emit_vsum4ubs (p, dest, dest, tmp1);
+  }
 }
-#endif
 
 static void
 powerpc_rule_signb (OrcCompiler *p, void *user, OrcInstruction *insn)
@@ -648,7 +732,7 @@ powerpc_rule_signl (OrcCompiler *p, void *user, OrcInstruction *insn)
 }
 
 static void
-powerpc_rule_select0wb (OrcCompiler *p, void *user, OrcInstruction *insn)
+powerpc_rule_select1wb (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int dest = ORC_DEST_ARG (p, insn, 0);
@@ -657,7 +741,7 @@ powerpc_rule_select0wb (OrcCompiler *p, void *user, OrcInstruction *insn)
 }
 
 static void
-powerpc_rule_select1wb (OrcCompiler *p, void *user, OrcInstruction *insn)
+powerpc_rule_select0wb (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int dest = ORC_DEST_ARG (p, insn, 0);
@@ -669,7 +753,7 @@ powerpc_rule_select1wb (OrcCompiler *p, void *user, OrcInstruction *insn)
 }
 
 static void
-powerpc_rule_select0lw (OrcCompiler *p, void *user, OrcInstruction *insn)
+powerpc_rule_select1lw (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int dest = ORC_DEST_ARG (p, insn, 0);
@@ -678,7 +762,7 @@ powerpc_rule_select0lw (OrcCompiler *p, void *user, OrcInstruction *insn)
 }
 
 static void
-powerpc_rule_select1lw (OrcCompiler *p, void *user, OrcInstruction *insn)
+powerpc_rule_select0lw (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
   int src1 = ORC_SRC_ARG (p, insn, 0);
   int dest = ORC_DEST_ARG (p, insn, 0);
@@ -689,6 +773,30 @@ powerpc_rule_select1lw (OrcCompiler *p, void *user, OrcInstruction *insn)
   powerpc_emit_vperm (p, dest, src1, src1, perm);
 }
 
+static void
+powerpc_rule_select1ql (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src1 = ORC_SRC_ARG (p, insn, 0);
+  int dest = ORC_DEST_ARG (p, insn, 0);
+  int perm;
+
+  perm = powerpc_get_constant_full (p, 0x04050607, 0x0c0d0e0f,
+		  0x14151617, 0x1c1d1e1f);
+  powerpc_emit_vperm (p, dest, src1, src1, perm);
+}
+
+static void
+powerpc_rule_select0ql (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src1 = ORC_SRC_ARG (p, insn, 0);
+  int dest = ORC_DEST_ARG (p, insn, 0);
+  int perm;
+
+  perm = powerpc_get_constant_full (p, 0x00010203, 0x08090a0b,
+		  0x10111213, 0x18191a1b);
+  powerpc_emit_vperm (p, dest, src1, src1, perm);
+}
+
 static void
 powerpc_rule_mergebw (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
@@ -697,8 +805,8 @@ powerpc_rule_mergebw (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest = ORC_DEST_ARG (p, insn, 0);
   int perm;
 
-  perm = powerpc_get_constant_full (p, 0x10001101, 0x12021303,
-      0x14041505, 0x16061707);
+  perm = powerpc_get_constant_full (p, 0x00100111, 0x02120313,
+      0x04140515, 0x06160717);
   powerpc_emit_vperm (p, dest, src1, src2, perm);
 }
 
@@ -710,8 +818,21 @@ powerpc_rule_mergewl (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest = ORC_DEST_ARG (p, insn, 0);
   int perm;
 
-  perm = powerpc_get_constant_full (p, 0x10110001, 0x12130203,
-      0x14150405, 0x16170607);
+  perm = powerpc_get_constant_full (p, 0x00011011, 0x02031213,
+      0x04051415, 0x06071617);
+  powerpc_emit_vperm (p, dest, src1, src2, perm);
+}
+
+static void
+powerpc_rule_mergelq (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src1 = ORC_SRC_ARG (p, insn, 0);
+  int src2 = ORC_SRC_ARG (p, insn, 1);
+  int dest = ORC_DEST_ARG (p, insn, 0);
+  int perm;
+
+  perm = powerpc_get_constant_full (p, 0x00010203, 0x10111213,
+      0x04050607, 0x14151617);
   powerpc_emit_vperm (p, dest, src1, src2, perm);
 }
 
@@ -899,6 +1020,58 @@ powerpc_rule_swapl (OrcCompiler *p, void *user, OrcInstruction *insn)
   powerpc_emit_vperm (p, dest, src1, src1, perm);
 }
 
+static void
+powerpc_rule_swapwl (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src1 = ORC_SRC_ARG (p, insn, 0);
+  int dest = ORC_DEST_ARG (p, insn, 0);
+  int perm;
+
+  perm = powerpc_get_constant_full (p, 0x02030001, 0x06070405,
+      0x0a0b0809, 0x0e0f0c0d);
+  powerpc_emit_vperm (p, dest, src1, src1, perm);
+}
+
+static void
+powerpc_rule_swaplq (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src1 = ORC_SRC_ARG (p, insn, 0);
+  int dest = ORC_DEST_ARG (p, insn, 0);
+  int perm;
+
+  perm = powerpc_get_constant_full (p, 0x04050607, 0x00010203,
+      0x0c0d0e0f, 0x08090a0b);
+  powerpc_emit_vperm (p, dest, src1, src1, perm);
+}
+
+static void
+powerpc_rule_swapq (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src1 = ORC_SRC_ARG (p, insn, 0);
+  int dest = ORC_DEST_ARG (p, insn, 0);
+  int perm;
+
+  perm = powerpc_get_constant_full (p, 0x07060504, 0x03020100,
+      0x0f0e0d0c, 0x0b0a0908);
+  powerpc_emit_vperm (p, dest, src1, src1, perm);
+}
+
+static void
+powerpc_rule_splitql (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int src1 = ORC_SRC_ARG (p, insn, 0);
+  int dest1 = ORC_DEST_ARG (p, insn, 0);
+  int dest2 = ORC_DEST_ARG (p, insn, 1);
+  int perm;
+
+  perm = powerpc_get_constant_full (p, 0x04050607, 0x0c0d0e0f,
+		  0x14151617, 0x1c1d1e1f);
+  powerpc_emit_vperm (p, dest1, src1, src1, perm);
+  perm = powerpc_get_constant_full (p, 0x00010203, 0x08090a0b,
+		  0x10111213, 0x18191a1b);
+  powerpc_emit_vperm (p, dest2, src1, src1, perm);
+}
+
 static void
 powerpc_rule_splitlw (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
@@ -907,10 +1080,10 @@ powerpc_rule_splitlw (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest2 = ORC_DEST_ARG (p, insn, 1);
   int perm;
 
-  powerpc_emit_vpkuwum (p, dest2, src1, src1);
+  powerpc_emit_vpkuwum (p, dest1, src1, src1);
   perm = powerpc_get_constant_full (p, 0x00010405, 0x08090c0d,
 		  0x10111415, 0x18191c1d);
-  powerpc_emit_vperm (p, dest1, src1, src1, perm);
+  powerpc_emit_vperm (p, dest2, src1, src1, perm);
 }
 
 static void
@@ -921,10 +1094,10 @@ powerpc_rule_splitwb (OrcCompiler *p, void *user, OrcInstruction *insn)
   int dest2 = ORC_DEST_ARG (p, insn, 1);
   int perm;
 
-  powerpc_emit_vpkuhum (p, dest2, src1, src1);
+  powerpc_emit_vpkuhum (p, dest1, src1, src1);
   perm = powerpc_get_constant_full (p, 0x00020406, 0x080a0c0e,
 		  0x10121416, 0x181a1c1e);
-  powerpc_emit_vperm (p, dest1, src1, src1, perm);
+  powerpc_emit_vperm (p, dest2, src1, src1, perm);
 }
 
 static void
@@ -1106,6 +1279,10 @@ orc_compiler_powerpc_register_rules (OrcTarget *target)
   REG(subusl);
   REG(xorl);
 
+  REG(andq);
+  REG(orq);
+  REG(xorq);
+
   REG(mullb);
   REG(mulhsb);
   REG(mulhub);
@@ -1131,9 +1308,9 @@ orc_compiler_powerpc_register_rules (OrcTarget *target)
   REG(mulswl);
   REG(muluwl);
 
-  //REG(accw);
+  REG(accw);
   REG(accl);
-  //REG(accsadubl);
+  REG(accsadubl);
 
   REG(signb);
   REG(signw);
@@ -1143,8 +1320,11 @@ orc_compiler_powerpc_register_rules (OrcTarget *target)
   REG(select1wb);
   REG(select0lw);
   REG(select1lw);
+  REG(select0ql);
+  REG(select1ql);
   REG(mergebw);
   REG(mergewl);
+  REG(mergelq);
 
   REG(absb);
   REG(absw);
@@ -1159,6 +1339,10 @@ orc_compiler_powerpc_register_rules (OrcTarget *target)
   REG(convql);
   REG(swapw);
   REG(swapl);
+  REG(swapwl);
+  REG(swapq);
+  REG(swaplq);
+  if (0) REG(splitql);
   REG(splitlw);
   REG(splitwb);
   REG(div255w);
@@ -1182,6 +1366,9 @@ orc_compiler_powerpc_register_rules (OrcTarget *target)
   orc_rule_register (rule_set, "loadw", powerpc_rule_loadX, NULL);
   orc_rule_register (rule_set, "loadl", powerpc_rule_loadX, NULL);
   orc_rule_register (rule_set, "loadq", powerpc_rule_loadX, NULL);
+  orc_rule_register (rule_set, "loadoffb", powerpc_rule_loadoffX, NULL);
+  orc_rule_register (rule_set, "loadoffw", powerpc_rule_loadoffX, NULL);
+  orc_rule_register (rule_set, "loadoffl", powerpc_rule_loadoffX, NULL);
   orc_rule_register (rule_set, "storeb", powerpc_rule_storeX, NULL);
   orc_rule_register (rule_set, "storew", powerpc_rule_storeX, NULL);
   orc_rule_register (rule_set, "storel", powerpc_rule_storeX, NULL);
@@ -1190,10 +1377,12 @@ orc_compiler_powerpc_register_rules (OrcTarget *target)
   orc_rule_register (rule_set, "andnb", powerpc_rule_andnX, NULL);
   orc_rule_register (rule_set, "andnw", powerpc_rule_andnX, NULL);
   orc_rule_register (rule_set, "andnl", powerpc_rule_andnX, NULL);
+  orc_rule_register (rule_set, "andnq", powerpc_rule_andnX, NULL);
 
   orc_rule_register (rule_set, "copyb", powerpc_rule_copyX, NULL);
   orc_rule_register (rule_set, "copyw", powerpc_rule_copyX, NULL);
   orc_rule_register (rule_set, "copyl", powerpc_rule_copyX, NULL);
+  orc_rule_register (rule_set, "copyq", powerpc_rule_copyX, NULL);
 
 }
 
