@@ -163,8 +163,8 @@ struct a ops[] = {
   { "minsl", "(a &lt; b) ? a : b", "signed minimum" },
   { "minul", "(a &lt; b) ? a : b", "unsigned minimum" },
   { "mulll", "a * b", "low bits of multiply" },
-  { "mulhsl", "(a * b) &gt;&gt; 8", "high bits of signed multiply" },
-  { "mulhul", "(a * b) &gt;&gt; 8", "high bits of unsigned multiply" },
+  { "mulhsl", "(a * b) &gt;&gt; 16", "high bits of signed multiply" },
+  { "mulhul", "(a * b) &gt;&gt; 16", "high bits of unsigned multiply" },
   { "orl", "a | b", "bitwise or" },
   { "shll", "a &lt;&lt; b", "shift left" },
   { "shrsl", "a &gt;&gt; b", "signed shift right" },
@@ -185,10 +185,10 @@ struct a ops[] = {
   { "convusswb", "clamp(a)", "convert unsigned to signed with saturation" },
   { "convuuswb", "clamp(a)", "convert unsigned to unsigned with saturation" },
   { "convlw", "a", "convert" },
-  { "convssslw", "clamp(a)", "convert signed to unsigned with saturation" },
-  { "convsuslw", "clamp(a)", "convert signed to signed with saturation" },
-  { "convusslw", "clamp(a)", "convert unsigned to unsigned with saturation" },
-  { "convuuslw", "clamp(a)", "convert unsigned to signed with saturation" },
+  { "convssslw", "clamp(a)", "convert signed to signed with saturation" },
+  { "convsuslw", "clamp(a)", "convert signed to unsigned with saturation" },
+  { "convusslw", "clamp(a)", "convert unsigned to signed with saturation" },
+  { "convuuslw", "clamp(a)", "convert unsigned to unsigned with saturation" },
   { "mulsbw", "a * b", "multiply signed" },
   { "mulubw", "a * b", "multiply unsigned" },
   { "mulswl", "a * b", "multiply signed" },
@@ -254,10 +254,10 @@ struct a ops[] = {
   { "convulq", "a", "unsigned convert" },
   { "convhwb", "a>>8", "shift and convert" },
   { "convhlw", "a>>16", "shift and convert" },
-  { "convsssql", "clamp(a)", "convert signed to unsigned with saturation" },
-  { "convsusql", "clamp(a)", "convert signed to signed with saturation" },
-  { "convussql", "clamp(a)", "convert unsigned to unsigned with saturation" },
-  { "convuusql", "clamp(a)", "convert unsigned to signed with saturation" },
+  { "convsssql", "clamp(a)", "convert signed to signed with saturation" },
+  { "convsusql", "clamp(a)", "convert signed to unsigned with saturation" },
+  { "convussql", "clamp(a)", "convert unsigned to signed with saturation" },
+  { "convuusql", "clamp(a)", "convert unsigned to unsigned with saturation" },
 
 };
 
