@@ -11,7 +11,7 @@
 #include <orc-test/orcprofile.h>
 
 
-#define ALIGN(ptr,n) ((void *)((unsigned long)(ptr) & (~(unsigned long)(n-1))))
+#define ALIGN(ptr,n) ((void *)((orc_intptr)(ptr) & (~(orc_intptr)(n-1))))
 
 int hot_src = TRUE;
 int hot_dest = TRUE;
@@ -90,7 +90,7 @@ main(int argc, char *argv[])
   orc_get_data_cache_sizes (&level1, &level2, &level3);
   if (level3 > 0) {
     max = (log(level3)/M_LN2 - 6.0) * 10 + 20;
-  } else if (level3 > 0) {
+  } else if (level2 > 0) {
     max = (log(level2)/M_LN2 - 6.0) * 10 + 20;
   } else {
     max = 140;
